#include "pch.h"
#include <Windows.h>
#include <iostream>
#include <signal.h>
#include "RtMidi.h"
#include "midifunctionsOfRtMidi.h"


#define RETURN_SUCCESS 0
#define RETURN_FAILURE 1

bool done;
static void finish(int /*ignore*/) { done = true; }

static RtMidiIn* midiin = 0;
static unsigned int portOfMidiIn = 0;
static RtMidiOut* midiout;
static unsigned int portOfMidiOut = 0;
std::vector<unsigned char> message(3);

__declspec(dllexport) int receiveMidi(int portNum) {
    std::vector<unsigned char> message;

    int nBytes;

    try {
        if (midiin == 0)
            midiin = new RtMidiIn();
    }
    catch (RtMidiError) {
        return RETURN_FAILURE;
    }

    if (portNum >= midiin->getPortCount()) {
        delete midiin;
        return RETURN_FAILURE;
    }

    // ignore sysex, timing, active sensing messages.
    midiin->ignoreTypes(true, true, true);

    // Install an interrupt handler function.
    done = false;
    (void)signal(SIGINT, finish);

    // Periodically check input queue.
    while (!done) {
        midiin->getMessage(&message);
        nBytes = (int)message.size();
        for (int i = 0; i < nBytes; i++)
            if (nBytes > 0) {
                int midiMessage = (message[0] << 16) + (message[1] << 8) + message[2];
                return midiMessage;
            }
        Sleep(1);
    }
    return RETURN_SUCCESS;
}

__declspec(dllexport) int setPortOfMidiIn(int portNum){
    try {
        if (midiin == 0)
            midiin = new RtMidiIn();
    }
    catch (std::exception) {
        return RETURN_FAILURE;
    }

    try {
        if (portNum >= midiin->getPortCount()) {
            delete midiin;
            return RETURN_FAILURE;
        }
        if (portOfMidiIn != portNum) {
            midiin->closePort();
            portOfMidiIn = portNum;
        }
        if (!midiin->isPortOpen()) {
            midiin->openPort(portNum);
            if (!midiin->isPortOpen()) {
                //delete midiin;
                return RETURN_FAILURE;
            }
        }
        return RETURN_SUCCESS;
    }
    catch (std::exception& e) {
        std::cout << e.what() << std::endl;
        delete midiin;
        return RETURN_FAILURE;
    }
}

__declspec(dllexport) int setPortOfMidiOut(int portNum){
    try {
        if (midiout == 0)
            midiout = new RtMidiOut();
    }
    catch (std::exception& e) {
        std::cout << e.what() << std::endl;
        return RETURN_FAILURE;
    }

    try {
        if (portNum >= midiout->getPortCount()) {
            delete midiin;
            return RETURN_FAILURE;
        }
        if (portOfMidiOut != portNum) {
            midiout->closePort();
            portOfMidiOut = portNum;
        }

        if (!midiout->isPortOpen()) {
            
            midiout->openPort(portNum);
           // std::cout << "midiout openPort in DLL " << portNum << std::endl;
            if (!midiout->isPortOpen()) {
               // delete midiout;
                return RETURN_FAILURE;
            }
            return RETURN_SUCCESS;
        }
    }
    catch (RtMidiError& error) {
        error.printMessage();
        delete midiout;
        return RETURN_FAILURE;
    }
}

__declspec(dllexport) void sendMidiMessage(unsigned int statusAndChannel, unsigned int data1, unsigned int data2) {
    try {
        if (midiout == 0)
            midiout = new RtMidiOut();
    }
    catch (RtMidiError& error) {
        error.printMessage();
        return;
    }

    message[0] = statusAndChannel;
    message[1] = data1;
    message[2] = data2;
    midiout->sendMessage(&message);

    return;
}